@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Marius L. Jøhndal
+  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Marius L. Jøhndal
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -50,8 +50,8 @@ static void version(void);
 static GKeyFile *_configuration_file_open(const gchar *rcfile);
 static void _configuration_file_close(GKeyFile *kf);
 #ifdef ENABLE_ID3LIB
-static int _id3_set(const gchar *filename, int clear, const gchar *lead_artist, 
-                    const gchar *content_group, const gchar *title, 
+static int _id3_set(const gchar *filename, int clear, const gchar *lead_artist,
+                    const gchar *content_group, const gchar *title,
                     const gchar *album, const gchar *content_type, const gchar *year,
                     const gchar *comment);
 static int _id3_check_and_set(const gchar *filename,
@@ -60,112 +60,102 @@ static int _id3_check_and_set(const gchar *filename,
 static int playlist_add(const gchar *playlist_file,
                         const gchar *media_file);
 
-static int verbose = 0;
-static int quiet = 0;
-static int new = 0;
-static int first_only = 0;
-static int resume = 0;
+static gboolean verbose = FALSE;
+static gboolean quiet = FALSE;
+static gboolean first_only = FALSE;
+static gboolean resume = FALSE;
+static gboolean debug = FALSE;
+static gboolean show_progress_bar = FALSE;
+static gboolean show_version = FALSE;
+static gboolean show_debug_info = FALSE;
+static gboolean new_only = FALSE;
+static gboolean list = FALSE;
+static gboolean catchup = FALSE;
+static gchar *rcfile = NULL;
+static gchar *filter_regex = NULL;
 
 int main(int argc, char **argv)
 {
-  char *rcfile = NULL;
   enum op op = OP_UPDATE;
-  int c, i, len;
+  int i, len;
   int ret = 0;
   gchar **groups;
   gchar *channeldir;
   GKeyFile *kf;
   struct channel_configuration *defaults;
   enclosure_filter *filter = NULL;
+  GError *error = NULL;
+  GOptionContext *context;
 
-  for (;;) {
-    int option_index = 0;
+  static GOptionEntry options[] =
+  {
+    {"catchup",      'c', 0, G_OPTION_ARG_NONE,     &catchup,           "catch up with channels and exit"},
+    {"list",         'l', 0, G_OPTION_ARG_NONE,     &list,              "list available enclosures that have not yet been downloaded and exit"},
+    {"version",      'V', 0, G_OPTION_ARG_NONE,     &show_version,      "print version and exit"},
 
-    static struct option long_options[] = {
-      {"catchup", 0, 0, 'c'},
-      {"rcfile", 1, 0, 'C'},
-      {"filter", 1, 0, 'f'},
-      {"first-only", 0, 0, '1'},
-      {"help", 0, 0, 'h'},
-      {"list", 0, 0, 'l'},
-      {"new-only", 0, 0, 'n'},
-      {"resume", 0, 0, 'r'},
-      {"verbose", 0, 0, 'v'},
-      {"version", 0, 0, 'V'},
-      {"quiet", 0, 0, 'q'},
-      {0, 0, 0, 0}
-    };
+    {"resume",       'r', 0, G_OPTION_ARG_NONE,     &resume,            "resume aborted downloads"},
+    {"rcfile",       'C', 0, G_OPTION_ARG_FILENAME, &rcfile,            "override the default configuration file name"},
 
-    c = getopt_long(argc, argv, "1cC:f:hlnqrvV", long_options, &option_index);
+    {"debug",        'd', 0, G_OPTION_ARG_NONE,     &show_debug_info,   "print connection debug information"},
+    {"verbose",      'v', 0, G_OPTION_ARG_NONE,     &verbose,           "print detailed progress information"},
+    {"progress-bar", 'p', 0, G_OPTION_ARG_NONE,     &show_progress_bar, "print progress bar"},
 
-    if (c == -1)
-      break;
-
-    switch (c) {
-    case 'c':
-      op = OP_CATCHUP;
-      break;
-
-    case 'C':
-      rcfile = g_strdup(optarg);
-      break;
-
-    case 'f':
+    {"new-only",     'n', 0, G_OPTION_ARG_NONE,     &new_only,          "only process new channels"},
+    {"quiet",        'q', 0, G_OPTION_ARG_NONE,     &quiet,             "only print error messages"},
+    {"first-only",   '1', 0, G_OPTION_ARG_NONE,     &first_only,        "only process the most recent item from each channel"},
 #ifdef ENABLE_GREGEX
-      filter = enclosure_filter_new(optarg, FALSE);
-#else
-      fprintf(stderr, "Regular expression filters not supported by this build of castget.\nPlease rebuild with support for GRegex.\n");
-      return 1;
-#endif
-      break;
+    {"filter",       'f', 0, G_OPTION_ARG_STRING,   &filter_regex,      "only process items whose enclosure names match a regular expression"},
+#endif /* ENABLE_GREGEX */
 
-    case 'l':
-      op = OP_LIST;
-      break;
+    { NULL }
+  };
 
-    case 'n':
-      new = 1;
-      break;
+  context = g_option_context_new("CHANNELS");
+  g_option_context_add_main_entries(context, options, NULL);
 
-    case 'r':
-      resume = 1;
-      break;
-
-    case '1':
-      first_only = 1;
-      break;
-
-    case 'v':
-      verbose = 1;
-      break;
-
-    case 'V':
-      version();
-      return 1;
-
-    case 'q':
-      quiet = 1;
-      break;
-
-    case 'h':
-    default:
-      usage();
-      return 1;
-    }
+  if (!g_option_context_parse(context, &argc, &argv, &error)) {
+    g_print("option parsing failed: %s\n", error->message);
+    exit(1);
   }
 
   /* Do some additional sanity checking of options. */
-  if (verbose && quiet) {
-    usage();
-    return 1;
+  if ((verbose && quiet) || (show_progress_bar && quiet)) {
+    g_print("option parsing failed: options are incompatible.\n");
+    exit(1);
   }
 
-  if (verbose && new) {
-    printf("Fetching new channels only...\n");
+  if ((catchup && list) || (catchup && show_version) || (list && show_version)) {
+    g_print("option parsing failed: --catchup, --list and --version options are incompatible.\n");
+    exit(1);
   }
 
+  /* Decide on the action to take */
+  if (show_version) {
+    version();
+    exit(0);
+  }
+
+  if (catchup)
+    op = OP_CATCHUP;
+
+  if (list)
+    op = OP_LIST;
+
+  if (filter_regex) {
+#ifdef ENABLE_GREGEX
+    filter = enclosure_filter_new(filter_regex, FALSE);
+    g_free(filter_regex);
+#else /* !ENABLE_GREGEX */
+    g_print("option parsing failed: filters not supported by this build.\n");
+    exit(1);
+#endif /* ENABLE_GREGEX */
+  }
+
+  if (verbose && new_only)
+    g_print("Fetching new channels only...\n");
+
   LIBXML_TEST_VERSION;
-        
+
   /* Build the channel directory path and ensure that it exists. */
   channeldir = g_build_filename(g_get_home_dir(), ".castget", NULL);
 
@@ -197,19 +187,19 @@ int main(int argc, char **argv)
     /* Perform actions. */
     if (optind < argc) {
       while (optind < argc)
-        _process_channel(channeldir, kf, argv[optind++], op, defaults, 
+        _process_channel(channeldir, kf, argv[optind++], op, defaults,
                          filter);
     } else {
       groups = g_key_file_get_groups(kf, NULL);
-      
+
       for (i = 0; groups[i]; i++)
         if (strcmp(groups[i], "*"))
-          _process_channel(channeldir, kf, groups[i], op, defaults, 
+          _process_channel(channeldir, kf, groups[i], op, defaults,
                            filter);
-      
+
       g_strfreev(groups);
     }
-  
+
     /* Clean up defaults. */
     if (defaults)
       channel_configuration_free(defaults);
@@ -226,41 +216,20 @@ int main(int argc, char **argv)
 
   if (kf)
     _configuration_file_close(kf);
-  
+
   xmlCleanupParser();
 
   return ret;
 }
 
-static void usage(void)
-{
-  g_printf("Usage: castget [-c|-l|-V|-h] [-v|-q] [-r] [-1] [-n] [identifier(s)]\n\n");
-  g_printf("  --catchup     -c    catch up with channels and exit.\n");
-  g_printf("  --list        -l    list available enclosures that have not yet been\n                      downloaded, and exit.\n");
-  g_printf("  --help        -h    display help and exit.\n");
-  g_printf("  --version     -V    output version information and exit.\n");
-  g_printf("\n");
-  g_printf("  --new-only    -n    restrict operation to new channels only.\n");
-  g_printf("  --first-only  -1    restrict operation to the most recent item in each channel\n                      only.\n");
-  g_printf("  --filter      -f    restrict operation to items whose enclosures have names\n                      matching the regular expression pattern.\n");
-  g_printf("\n");
-  g_printf("  --resume      -r    resume aborted downloads.\n");
-  g_printf("  --quiet       -q    do not print anything except error messages.\n");
-  g_printf("  --verbose     -v    print detailed progress information.\n");
-  g_printf("  --rcfile      -C    override the default filename for the configuration file.\n");
-  g_printf("\n");
-  g_printf("The identifiers identifies the channels affected by the selected operation.\n");
-  g_printf("If no identifier is supplied all channels are affect.\n");
-}
-
 static void version(void)
 {
   g_printf("%s %s\n", PACKAGE, VERSION);
-  g_printf("Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Marius L. Jøhndal <mariuslj at ifi.uio.no>\n");
+  g_printf("Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Marius L. Jøhndal <mariuslj at ifi.uio.no>\n");
 }
 
-static void update_callback(void *user_data, channel_action action, 
-                            channel_info *channel_info, enclosure *enclosure, 
+static void update_callback(void *user_data, channel_action action,
+                            channel_info *channel_info, enclosure *enclosure,
                             const gchar *filename)
 {
   struct channel_configuration *c = (struct channel_configuration *)user_data;
@@ -270,29 +239,29 @@ static void update_callback(void *user_data, channel_action action,
     if (!quiet)
       g_printf("Updating channel %s...\n", c->identifier);
     break;
-    
+
   case CCA_RSS_DOWNLOAD_END:
     break;
-    
+
   case CCA_ENCLOSURE_DOWNLOAD_START:
     g_assert(channel_info);
     g_assert(enclosure);
-    
+
     if (verbose) {
       if (enclosure->length > 1024*1024*1024) {
-        g_printf(" * Downloading %s (%.1f GB) from %s\n", 
-                 enclosure->filename, (float)enclosure->length / (1024.0*1024.0*1024.0), 
+        g_printf(" * Downloading %s (%.1f GB) from %s\n",
+                 enclosure->filename, (float)enclosure->length / (1024.0*1024.0*1024.0),
                  channel_info->title);
       } else if (enclosure->length > 1024*1024) {
-        g_printf(" * Downloading %s (%.1f MB) from %s\n", 
+        g_printf(" * Downloading %s (%.1f MB) from %s\n",
                  enclosure->filename, (float)enclosure->length / (1024.0*1024.0),
                  channel_info->title);
       } else if (enclosure->length > 1024) {
-        g_printf(" * Downloading %s (%.1f kB) from %s\n", 
+        g_printf(" * Downloading %s (%.1f kB) from %s\n",
                  enclosure->filename, (float)enclosure->length / 1024.0,
                  channel_info->title);
       } else if (enclosure->length > 0) {
-        g_printf(" * Downloading %s (%ld bytes) from %s\n", 
+        g_printf(" * Downloading %s (%ld bytes) from %s\n",
                  enclosure->filename, enclosure->length,
                  channel_info->title);
       } else {
@@ -302,12 +271,12 @@ static void update_callback(void *user_data, channel_action action,
       }
     }
     break;
-    
+
   case CCA_ENCLOSURE_DOWNLOAD_END:
     g_assert(channel_info);
     g_assert(enclosure);
     g_assert(filename);
-    
+
     /* Set media tags. */
     if (enclosure->type && !strcmp(enclosure->type, "audio/mpeg")) {
 #ifdef ENABLE_ID3LIB
@@ -315,13 +284,13 @@ static void update_callback(void *user_data, channel_action action,
         fprintf(stderr, "Error setting ID3 tag for file %s.\n", filename);
 #endif /* ENABLE_ID3LIB */
     }
-    
+
     /* Update playlist. */
     if (c->playlist) {
       playlist_add(c->playlist, filename);
-      
+
       if (verbose)
-        printf(" * Added downloaded enclosure %s to playlist %s.\n", 
+        printf(" * Added downloaded enclosure %s to playlist %s.\n",
                filename, c->playlist);
     }
     break;
@@ -338,19 +307,19 @@ static void catchup_callback(void *user_data, channel_action action, channel_inf
     if (!quiet)
       g_printf("Catching up with channel %s...\n", c->identifier);
     break;
-    
+
   case CCA_RSS_DOWNLOAD_END:
     break;
 
   case CCA_ENCLOSURE_DOWNLOAD_START:
     g_assert(channel_info);
     g_assert(enclosure);
-    
+
     if (verbose)
       g_printf("Catching up on %s (%ld bytes) from %s\n", enclosure->url, enclosure->length,
                channel_info->title);
     break;
-    
+
   case CCA_ENCLOSURE_DOWNLOAD_END:
     break;
   }
@@ -365,41 +334,41 @@ static void list_callback(void *user_data, channel_action action, channel_info *
   case CCA_RSS_DOWNLOAD_START:
     g_printf("Listing channel %s...\n", c->identifier);
     break;
-    
+
   case CCA_RSS_DOWNLOAD_END:
     break;
-    
+
   case CCA_ENCLOSURE_DOWNLOAD_START:
     g_assert(channel_info);
     g_assert(enclosure);
-    
+
     if (enclosure->length > 1024*1024*1024) {
-      g_printf(" * %s (%.1f GB) from %s\n", 
-               enclosure->filename, (float)enclosure->length / (1024.0*1024.0*1024.0), 
+      g_printf(" * %s (%.1f GB) from %s\n",
+               enclosure->filename, (float)enclosure->length / (1024.0*1024.0*1024.0),
                channel_info->title);
     } else if (enclosure->length > 1024*1024) {
-      g_printf(" * %s (%.1f MB) from %s\n", 
+      g_printf(" * %s (%.1f MB) from %s\n",
                enclosure->filename, (float)enclosure->length / (1024.0*1024.0),
                channel_info->title);
     } else if (enclosure->length > 1024) {
-      g_printf(" * %s (%.1f kB) from %s\n", 
+      g_printf(" * %s (%.1f kB) from %s\n",
                enclosure->filename, (float)enclosure->length / 1024.0,
                channel_info->title);
     } else if (enclosure->length > 0) {
-      g_printf(" * %s (%ld bytes) from %s\n", 
+      g_printf(" * %s (%ld bytes) from %s\n",
                enclosure->filename, enclosure->length, channel_info->title);
     } else {
       g_printf(" * %s from %s\n", enclosure->filename, channel_info->title);
     }
-    
+
     break;
-    
+
   case CCA_ENCLOSURE_DOWNLOAD_END:
     break;
   }
 }
 
-static int _process_channel(const gchar *channel_directory, GKeyFile *kf, const char *identifier, 
+static int _process_channel(const gchar *channel_directory, GKeyFile *kf, const char *identifier,
                             enum op op, struct channel_configuration *defaults,
                             enclosure_filter *filter)
 {
@@ -424,7 +393,7 @@ static int _process_channel(const gchar *channel_directory, GKeyFile *kf, const
   /* Check that mandatory keys were set. */
   if (!channel_configuration->url) {
     fprintf(stderr, "No feed URL set for channel %s.\n", identifier);
-      
+
     channel_configuration_free(channel_configuration);
     return -1;
   }
@@ -441,7 +410,7 @@ static int _process_channel(const gchar *channel_directory, GKeyFile *kf, const
   channel_file = g_build_filename(channel_directory, channel_filename, NULL);
   g_free(channel_filename);
 
-  if (new && access(channel_file, F_OK) == 0) {
+  if (new_only && access(channel_file, F_OK) == 0) {
     /* If we are only fetching new channels, skip the channel if there is
        already a channel file present. */
 
@@ -449,7 +418,7 @@ static int _process_channel(const gchar *channel_directory, GKeyFile *kf, const
     return 0;
   }
 
-  c = channel_new(channel_configuration->url, channel_file, 
+  c = channel_new(channel_configuration->url, channel_file,
                   channel_configuration->spool_directory, resume);
   g_free(channel_file);
 
@@ -463,26 +432,26 @@ static int _process_channel(const gchar *channel_directory, GKeyFile *kf, const
   /* Set up per-channel filter unless overridden on the command
      line. */
   if (!filter && channel_configuration->regex_filter) {
-    per_channel_filter = 
+    per_channel_filter =
       enclosure_filter_new(channel_configuration->regex_filter, FALSE);
 
     filter = per_channel_filter;
   }
-    
+
   switch (op) {
   case OP_UPDATE:
-    channel_update(c, channel_configuration, update_callback, 0, 0, 
-                   first_only, resume, filter);
+    channel_update(c, channel_configuration, update_callback, 0, 0,
+                   first_only, resume, filter, debug, show_progress_bar);
     break;
-            
+
   case OP_CATCHUP:
-    channel_update(c, channel_configuration, catchup_callback, 1, 0, 
-                   first_only, 0, filter);
+    channel_update(c, channel_configuration, catchup_callback, 1, 0,
+                   first_only, 0, filter, debug, show_progress_bar);
     break;
-            
+
   case OP_LIST:
-    channel_update(c, channel_configuration, list_callback, 1, 1, first_only, 
-                   0, filter);
+    channel_update(c, channel_configuration, list_callback, 1, 1, first_only,
+                   0, filter, debug, show_progress_bar);
     break;
   }
 
@@ -531,11 +500,11 @@ static int _id3_find_and_set_frame(ID3Tag *tag, ID3_FrameID id, const char *valu
   if (value && strlen(value) > 0) {
     frame = ID3Frame_NewID(id);
     g_assert(frame);
-    
+
     ID3Tag_AttachFrame(tag, frame);
-    
+
     field = ID3Frame_GetField(frame, ID3FN_TEXT);
-    
+
     if (field)
       ID3Field_SetASCII(field, value); //TODO: UTF8
     else
@@ -545,8 +514,8 @@ static int _id3_find_and_set_frame(ID3Tag *tag, ID3_FrameID id, const char *valu
   return 0;
 }
 
-static int _id3_set(const gchar *filename, int clear, const gchar *lead_artist, 
-                    const gchar *content_group, const gchar *title, const gchar *album, 
+static int _id3_set(const gchar *filename, int clear, const gchar *lead_artist,
+                    const gchar *content_group, const gchar *title, const gchar *album,
                     const gchar *content_type, const gchar *year, const gchar *comment)
 {
   int errors = 0;
@@ -622,10 +591,10 @@ static int _id3_set(const gchar *filename, int clear, const gchar *lead_artist,
 static int _id3_check_and_set(const gchar *filename,
                               const struct channel_configuration *cfg)
 {
-  if (cfg->id3_lead_artist || cfg->id3_content_group || cfg->id3_title || 
+  if (cfg->id3_lead_artist || cfg->id3_content_group || cfg->id3_title ||
       cfg->id3_album || cfg->id3_content_type || cfg->id3_year || cfg->id3_comment)
-    return _id3_set(filename, 0, cfg->id3_lead_artist, cfg->id3_content_group, 
-                    cfg->id3_title, cfg->id3_album, cfg->id3_content_type, 
+    return _id3_set(filename, 0, cfg->id3_lead_artist, cfg->id3_content_group,
+                    cfg->id3_title, cfg->id3_album, cfg->id3_content_type,
                     cfg->id3_year, cfg->id3_comment);
   else
     return 0;
@@ -646,17 +615,7 @@ static int playlist_add(const gchar *playlist_file,
     return -1;
   }
 
-  fprintf(f, "%s\n", media_file); 
+  fprintf(f, "%s\n", media_file);
   fclose(f);
   return 0;
 }
-
-
-/* 
-   Local Variables:
-   mode:c
-   indent-tabs-mode:nil
-   c-basic-offset:2
-   coding:utf-8
-   End:
-*/
